use std::{cmp::Ordering, collections::HashMap, ops::RangeInclusive, sync::LazyLock, usize};

use log::warn;

const JOKER: char = 'J';
static LETTERS_WITHOUT_JOKER: &[char] = &['T', 'J', 'Q', 'K', 'A'];
static LETTERS: &[char] = &['T', 'J', 'Q', 'K', 'A'];
const NUMBERS: RangeInclusive<u32> = 0..=9;

pub type StrengthOfSymbols = HashMap<char, usize>;

fn numbers_in_chars() -> impl Iterator<Item = char> {
    NUMBERS
        .into_iter()
        .map(|number| std::char::from_digit(number, 10).unwrap())
}

//A hand consists of five cards labeled one of A, K, Q, J, T, 9, 8, 7, 6, 5, 4, 3, or 2.
//The relative strength of each card follows this order, where A is the highest and 2 is the lowest.
pub static LABEL_ORDERING: LazyLock<StrengthOfSymbols> = LazyLock::new(|| {
    numbers_in_chars()
        .chain(LETTERS.into_iter().copied())
        .into_iter()
        .enumerate()
        .map(|(index, character)| (character, index))
        .collect()
});

pub static LABEL_ORDERING_WITH_JOKER: LazyLock<StrengthOfSymbols> = LazyLock::new(|| {
    std::iter::once(JOKER)
        .chain(numbers_in_chars())
        .chain(LETTERS_WITHOUT_JOKER.into_iter().copied())
        .into_iter()
        .enumerate()
        .map(|(index, character)| (character, index))
        .collect()
});

pub fn compare_along_str(
    left: &str,
    right: &str,
    symbols_to_strength: &StrengthOfSymbols,
) -> Ordering {
    for (left_char, right_char) in left.chars().zip(right.chars()) {
        let left_rank_value = symbols_to_strength
            .get(&left_char)
            .unwrap_or_else(|| panic!("left char ({}) is not valid", left_char));
        let right_rank_value = symbols_to_strength
            .get(&right_char)
            .unwrap_or_else(|| panic!("right char ({}) is not valid", right_char));

        let current_ordering = left_rank_value.cmp(&right_rank_value);

        let found_difference = current_ordering != Ordering::Equal;
        if found_difference {
            return current_ordering;
        }
    }
    let (left_len, right_len) = (left.len(), right.len());
    warn!(
        "Left and right seem to have no difference in strength\n\
        Now comparing via lenght ({}) ({}). By same length, lefte wins !",
        left_len, right_len
    );
    left_len.cmp(&right_len)
}
